package jp.hishidama.swing.undo;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEditSupport;

import jp.hishidama.swing.action.ExAction;

/**
 * UndoManager[eBeB[.
 * <p>
 * UndoManagerɊւANZ[^[L[ݒ肷B
 * </p>
 *
 * @author <a target="hishidama"
 *         href="http://www.ne.jp/asahi/hishidama/home/tech/soft/java/swing.html"
 *         >Ђ</a>
 * @since 2009.04.12
 * @version 2009.04.26 UndoManagerRXgN^[Ŏw肷悤ύX
 */
public class UndoManagerUtil {

	private UndoAction actionUndo;
	private RedoAction actionRedo;

	protected UndoManager um;

	/**
	 * RXgN^[.
	 */
	public UndoManagerUtil() {
		um = new UndoManager();
	}

	/**
	 * RXgN^[.
	 *
	 * @param um
	 *            UndoManager
	 */
	public UndoManagerUtil(UndoManager um) {
		this.um = um;
	}

	/**
	 * UndoManager擾.
	 *
	 * @return UndoManager
	 */
	public UndoManager getUndoManager() {
		return um;
	}

	/**
	 * UndoManagerݒ.
	 * <p>
	 * eLXgR|[lgUndoManagero^ƋɁAANZ[^[L[ݒ肷B
	 * </p>
	 *
	 * @param tc
	 *            ݒΏۃeLXgR|[lg
	 * @see #setAcceleratorKey(JComponent)
	 */
	public void installTo(JTextComponent tc) {
		Document doc = tc.getDocument();
		doc.addUndoableEditListener(um);

		setAcceleratorKey(tc);
	}

	/**
	 * UndoManager폜.
	 *
	 * @param tc
	 *            UndoManager폜eLXgR|[lg
	 */
	public void uninstall(JTextComponent tc) {
		Document doc = tc.getDocument();
		doc.removeUndoableEditListener(um);

		removeAcceleratorKey(tc);
	}

	/**
	 * UndoManagerݒ.
	 * <p>
	 * UndoableEditSupportUndoManagero^ƋɁAR|[lgɃANZ[^[L[ݒ肷B
	 * </p>
	 *
	 * @param c
	 *            ݒΏۃR|[lg
	 * @param us
	 *            UndoableEditSupport
	 * @see #setAcceleratorKey(JComponent)
	 * @since 2009.04.25
	 */
	public void installTo(JComponent c, UndoableEditSupport us) {
		us.addUndoableEditListener(um);

		setAcceleratorKey(c);
	}

	/**
	 * UndoManager폜.
	 *
	 * @param c
	 *            UndoManager폜R|[lg
	 * @param us
	 *            UndoableEditSupport
	 * @since 2009.04.25
	 */
	public void uninstall(JComponent c, UndoableEditSupport us) {
		us.removeUndoableEditListener(um);

		removeAcceleratorKey(c);
	}

	/**
	 * ANZ[^[L[ݒ.
	 * <p>
	 * Ctrl+ZECtrl+YUNDOEREDO蓖ĂB
	 * </p>
	 *
	 * @param tc
	 *            ݒΏۃeLXgR|[lg
	 * @see #createDefaultUndoAction()
	 * @see #createDefaultRedoAction()
	 * @since 2009.04.25
	 */
	public void setAcceleratorKey(JComponent tc) {
		if (actionUndo == null) {
			actionUndo = createDefaultUndoAction();
		}
		if (actionRedo == null) {
			actionRedo = createDefaultRedoAction();
		}

		InputMap im = tc.getInputMap();
		im.put(actionUndo.getAcceleratorKey(), UndoAction_NAME);
		im.put(actionRedo.getAcceleratorKey(), RedoAction_NAME);
		ActionMap am = tc.getActionMap();
		am.put(UndoAction_NAME, actionUndo);
		am.put(RedoAction_NAME, actionRedo);
	}

	/**
	 * ANZ[^[L[.
	 *
	 * @param tc
	 *            eLXgR|[lg
	 * @since 2009.04.25
	 */
	public static void removeAcceleratorKey(JComponent tc) {
		InputMap im = tc.getInputMap();
		ActionMap am = tc.getActionMap();
		removeAction(im, am, UndoAction_NAME);
		removeAction(im, am, RedoAction_NAME);
	}

	protected static void removeAction(InputMap im, ActionMap am, String name) {
		Action a = am.get(name);
		if (a != null) {
			KeyStroke ks = (KeyStroke) a.getValue(Action.ACCELERATOR_KEY);
			if (ks != null) {
				im.remove(ks);
			}
			am.remove(name);
		}
	}

	/**
	 * UNDOj[ǉ.
	 *
	 * @param pmenu
	 *            |bvAbvj[
	 * @since 2009.04.26
	 */
	public void addMenuUndo(JPopupMenu pmenu) {
		if (actionUndo == null) {
			actionUndo = createDefaultUndoAction();
		}
		if (actionRedo == null) {
			actionRedo = createDefaultRedoAction();
		}
		pmenu.add(actionUndo);
		pmenu.add(actionRedo);
	}

	/**
	 * UNDOANV.
	 *
	 * @return ANVCX^X
	 */
	public UndoAction createDefaultUndoAction() {
		return new UndoAction(um);
	}

	/**
	 * REDOANV.
	 *
	 * @return ANVCX^X
	 */
	public RedoAction createDefaultRedoAction() {
		return new RedoAction(um);
	}

	/** UNDOANV */
	public static final String UndoAction_NAME = "ɖ߂(U)";
	/** REDOANV */
	public static final String RedoAction_NAME = "蒼(R)";

	/**
	 * UNDOANV
	 */
	public static class UndoAction extends ExAction {
		private static final long serialVersionUID = 3893649109654476856L;

		protected UndoManager um;

		/**
		 * RXgN^[.
		 *
		 * @param um
		 *            UNDOsۂɎgUndoManager
		 */
		public UndoAction(UndoManager um) {
			super(UndoAction_NAME);
			putMnemonicKey('U');
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_Z,
					KeyEvent.CTRL_DOWN_MASK));
			this.um = um;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (um.canUndo()) {
				um.undo();
			}
		}

		@Override
		public boolean isEnabled() {
			return um.canUndo();
		}
	}

	/**
	 * REDOANV
	 */
	public static class RedoAction extends ExAction {
		private static final long serialVersionUID = -7720240262168030520L;

		protected UndoManager um;

		/**
		 * RXgN^[.
		 *
		 * @param um
		 *            REDOsۂɎgUndoManager
		 */
		public RedoAction(UndoManager um) {
			super(RedoAction_NAME);
			putMnemonicKey('R');
			putAcceleratorKey(KeyStroke.getKeyStroke(KeyEvent.VK_Y,
					KeyEvent.CTRL_DOWN_MASK));
			this.um = um;
		}

		@Override
		public void actionPerformed(ActionEvent e) {
			if (um.canRedo()) {
				um.redo();
			}
		}

		@Override
		public boolean isEnabled() {
			return um.canRedo();
		}
	}
}
